/**
 * Copyright (c) 2006 - 2010 Smaxe Ltd (www.smaxe.com).
 * All rights reserved.
 */

import com.smaxe.me.uv.ObjectEncoding;
import com.smaxe.me.uv.client.IVideo;
import com.smaxe.me.uv.client.License;
import com.smaxe.me.uv.client.NetConnection;
import com.smaxe.me.uv.client.NetStream;
import com.smaxe.me.uv.client.video.EmptyVideo;
import com.smaxe.me.uv.stream.MediaData;

import java.io.IOException;
import java.io.InputStream;
import java.util.Hashtable;

import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.TextBox;
import javax.microedition.midlet.MIDlet;

/**
 * <code>ExConnectAndPlayMidlet</code>
 *
 * @author Andrei Sochirca
 */
public final class ExConnectAndPlayMidlet extends MIDlet implements CommandListener
{
    /**
     * <code>NetConnectionListener</code> - {@link NetConnection} listener implementation.
     */
    public class NetConnectionListener implements NetConnection.IListener
    {
        /**
         * Constructor.
         */
        public NetConnectionListener()
        {
        }

        public void onAsyncError(final NetConnection source, final String message, final Exception e)
        {
            System.out.println("NetConnection#onAsyncError: " + message + " " + e);
        }

        public void onIOError(final NetConnection source, final String message)
        {
            System.out.println("NetConnection#onIOError: " + message);
        }

        public void onNetStatus(final NetConnection source, final Hashtable info)
        {
            System.out.println("NetConnection#onNetStatus: " + info);

            final Object code = info.get("code");

            if (NetConnection.CONNECT_SUCCESS.equals(code))
            {
            }
            else
            {
                disconnected = true;
            }
        }

        public void onSecurityError(NetConnection arg0, String arg1)
        {
        }
    }

    /**
     * <code>PlayVideo</code> - {@link IVideo} implementation that logs
     * incoming audio/video packets.
     */
    private class PlayVideo extends EmptyVideo implements IVideo
    {
        /**
         * Constructor.
         */
        public PlayVideo()
        {
        }

        // IVideo implementation

        public void onAudioData(final MediaData data)
        {
            System.out.println("onAudioData");

            final int tag = data.tag();

            final byte[] encodedAudio = new byte[data.size() - 1];

            try
            {
                InputStream is = data.read();

                // tag byte
                is.read();

                // read encoded audio
                is.read(encodedAudio);
            }
            catch (IOException e) {}

            if (getAudioFormat(tag) == 0x02) // MP3
            {
                // TODO decode it
            }
        }

        public void onVideoData(final MediaData data)
        {
            System.out.println("onVideoData");

            final int tag = data.tag();

            final byte[] encodedVideo = new byte[data.size() - 1];

            try
            {
                InputStream is = data.read();

                // tag byte
                is.read();

                // read encoded video
                is.read(encodedVideo);
            }
            catch (IOException e) {}

            if (getVideoCodec(tag) == 0x02) // SORENSON_H263
            {
                // TODO decode it
            }
        }

        public void onFlvData(final MediaData data)
        {
        }

        public void onCuePoint(final Object data)
        {
            System.out.println("onCuePoint: " + data);
        }

        public void onMetaData(final Object data)
        {
            System.out.println("onMetaData: " + data);
        }

        // inner use methods
        
        /**
         * @param tag
         * @return format encoded in the <code>tag</code>
         */
        public int getAudioFormat(final int tag)
        {
            return (tag >> 4) & 0x0F;
        }

        /**
         * @param tag
         * @return video codec encoded in the <code>tag</code>
         */
        public int getVideoCodec(final int tag)
        {
            return tag & 0x0F;
        }
    }

    // fields
    private Command exitCommand;
    private Command connectAndPlayCommand;

    private volatile static boolean disconnected = false;

    private TextBox textBox;

    /**
     * Constructor.
     */
    public ExConnectAndPlayMidlet()
    {
        // NOTE:
        // you can get Evaluation Key at:
        // http://www.smaxe.com/order.jsf#request_evaluation_key
        // or buy at:
        // http://www.smaxe.com/order.jsf
        License.setKey("SET-YOUR-KEY");
        
        
        connectAndPlayCommand = new Command("Connect & Play", Command.SCREEN, 11);
        exitCommand = new Command("Exit", Command.EXIT, 1);

        textBox = new TextBox("Sample MIDlet", "Sample", 15, 0);
        textBox.addCommand(connectAndPlayCommand);
        textBox.addCommand(exitCommand);
        textBox.setCommandListener(this);
    }

    protected void startApp()
    {
        Display.getDisplay(this).setCurrent(textBox);
    }

    protected void pauseApp()
    {
    }

    protected void destroyApp(boolean u)
    {
    }

    public void commandAction(Command c, Displayable d)
    {
        if (c == exitCommand)
        {
            destroyApp(false);
            notifyDestroyed();
        }
        else
        if (c == connectAndPlayCommand)
        {
            new Thread(new Runnable()
            {
                public void run()
                {
                    final NetConnection connection = new NetConnection();

                    connection.objectEncoding(ObjectEncoding.AMF3);

                    // configure NetConnection
                    connection.configuration().put(NetConnection.Configuration.INACTIVITY_TIMEOUT, new Integer(2 * 60) /*seconds*/);

                    connection.configuration().put(NetConnection.Configuration.SOCKET_CONNECTION_KEEP_ALIVE, new Integer(1));
                    connection.configuration().put(NetConnection.Configuration.SOCKET_CONNECTION_LINGER, new Integer(5));

                    connection.addEventListener(new NetConnectionListener());

                    try
                    {
                        connection.connect("rtmp://localhost:1935/live", null /*arguments*/);
                    }
                    catch (Exception e)
                    {
                        e.printStackTrace();
                    }

                    // wait till connected
                    while (!connection.connected() && !disconnected)
                    {
                        try
                        {
                            Thread.sleep(100);
                        }
                        catch (Exception e) {/*ignore*/}
                    }

                    if (!disconnected)
                    {
                        NetStream stream = new NetStream(connection);
                        
                        stream.play(new PlayVideo(), new Object[] {"streamName"});
                    }
                }
            }).start();
        }
    }
}