/**
 * Copyright (c) 2006 - 2008 Smaxe Ltd (www.smaxe.com).
 * All rights reserved.
 */

import com.smaxe.me.uv.ObjectEncoding;
import com.smaxe.me.uv.client.NetConnection;
import com.smaxe.me.uv.client.License;
import com.smaxe.me.uv.client.SharedObject;
import com.smaxe.me.uv.invoker.IMethodInvoker;

import java.util.Hashtable;
import java.util.Vector;

/**
 * <code>ExMeConnectRemoteSharedObject</code> - {@link NetConnection} usage example.
 * <p> Note: The example shows how to connect to the remote shared object, set shared
 * object values.
 * 
 * @author Andrei Sochirca
 */
public final class ExMeConnectRemoteSharedObject extends Object
{
    /**
     * Entry point.
     * 
     * @param args
     */
    public static void main(final String[] args)
    {
        // NOTE:
        // you can get Evaluation Key at:
        // http://www.smaxe.com/order.jsf#request_evaluation_key
        // or buy at:
        // http://www.smaxe.com/order.jsf
        License.setKey("SET-YOUR-KEY");
        
        ExMeConnectRemoteSharedObject app = new ExMeConnectRemoteSharedObject();
        
        app.start();
    }
    
    /**
     * <code>NetConnectionListener</code> - {@link NetConnection} listener implementation.
     */
    public class NetConnectionListener extends NetConnection.ListenerAdapter
    {
        /**
         * Constructor.
         */
        public NetConnectionListener()
        {
        }
        
        public void onAsyncError(final NetConnection source, final String message, final Exception e)
        {
            System.out.println("NetConnection#onAsyncError: " + message + " " + e);
        }
        
        public void onIOError(final NetConnection source, final String message)
        {
            System.out.println("NetConnection#onIOError: " + message);
        }
        
        public void onNetStatus(final NetConnection source, final Hashtable info)
        {
            System.out.println("NetConnection#onNetStatus: " + info);
            
            final Object code = info.get("code");
            
            if (NetConnection.CONNECT_SUCCESS.equals(code))
            {
            }
            else
            {
                disconnected = true;
            }
        }
    }
    
    /**
     * <code>MethodInvoker</code> - method invoker
     */
    private class MethodInvoker extends Object implements IMethodInvoker
    {
        /**
         * Constructor.
         */
        public MethodInvoker()
        {
        }
        
        public void invoke(final Object o, final String method, final ICallback callback, final Object[] args)
        {
            // implement method invocations
            System.out.println("Invoke method: " + method);
        }
    }
    
    // fields
    private volatile boolean disconnected = false;
    
    /**
     * Constructor.
     */
    public ExMeConnectRemoteSharedObject()
    {
    }
    
    /**
     * Starts the example.
     */
    public void start()
    {
        final NetConnection connection = new NetConnection();
        
        connection.objectEncoding(ObjectEncoding.AMF3);
        
        connection.configuration().put(NetConnection.Configuration.METHOD_INVOKER, new MethodInvoker());
        
        connection.addEventListener(new NetConnectionListener());
        
        connection.connect("rtmp://localhost:1935/app", null) /*arguments*/;
        
        // wait till connected
        while (!connection.connected() && !disconnected)
        {
            try
            {
                Thread.sleep(100);
            }
            catch (Exception e) {/*ignore*/}
        }
        
        if (!disconnected)
        {
            SharedObject so = new SharedObject("mySharedObject");
            
            so.addEventListener(new SharedObject.ListenerAdapter()
            {
                public void onSync(SharedObject source, final Vector changeList)
                {
                    for (int i = 0, n = changeList.size(); i < n; i++)
                    {
                        final SharedObject.Change change = (SharedObject.Change) changeList.elementAt(i);
                        
                        switch (change.code)
                        {
                            case SharedObject.Change.CONNECT:
                            {
                                System.out.println("SharedObject " + source.getName() + " : " + "connected");
                                
                                source.data().put("attr1", new Integer(123));
                                
                            } break;
                            case SharedObject.Change.CHANGE:
                            {
                                System.out.println("SharedObject " + source.getName() + " : " +
                                        "attribute " + change.attribute + " changed from " + change.oldValue + " to " + change.newValue);
                            } break;
                            case SharedObject.Change.STATUS:
                            {
                                
                            } break;
                        }
                    }
                }
            });
            
            so.connect(connection, so.getName());
        }
        
        while (!disconnected)
        {
            try
            {
                Thread.sleep(100);
            }
            catch (Exception e) {/*ignore*/}
        }
        
        connection.close();
    }
}